// -*- C++ -*-
/*!
 * @file  rtc_3DShapeControl.cpp
 * @brief 3D images-shape control
 * @date $Date$
 *
 * @author EAST JAPAN ACCOUNTING CENTER CO.,LTD.
 *
 * BSD 3-clause "New" or "Revised" License
 *
 * $Id$
 */

#include "rtc_3DShapeControl.h"

// Module specification
// <rtc-template block="module_spec">
static const char* rtc_3dshapecontrol_spec[] =
  {
    "implementation_id", "rtc_3DShapeControl",
    "type_name",         "rtc_3DShapeControl",
    "description",       "3D images-shape control",
    "version",           "1.0",
    "vendor",            "EAC",
    "category",          "Image",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
rtc_3DShapeControl::rtc_3DShapeControl(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_img_cameraIn("ImgIn_Camera", m_img_camera),
    m_img_captureIn("ImgIn_Capture", m_img_capture),
    m_img_cloudIn("ImgIn_Cloud", m_img_cloud),
    m_sys_modeIn("ModeIn_3DSystem", m_sys_mode),
    m_play_modeIn("StartIn_3DSystem", m_play_mode),
    m_img_2dcdp1Out("ImgOut_2dcdp1", m_img_2dcdp1),
    m_img_2dcdp2Out("ImgOut_2dcdp2", m_img_2dcdp2),
    m_Control3DShapeServPort("Control3DShapeServ")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
rtc_3DShapeControl::~rtc_3DShapeControl()
{
}

//add to code
/*=================================================================================================
  Variables definition
 ================================================================================================*/


/*!
 * Initialization of the 3DShapeControl RTC
 */
RTC::ReturnCode_t rtc_3DShapeControl::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("ImgIn_Camera", m_img_cameraIn);
  addInPort("ImgIn_Capture", m_img_captureIn);
  addInPort("ImgIn_Cloud", m_img_cloudIn);
  addInPort("ModeIn_3DSystem", m_sys_modeIn);
  addInPort("StartIn_3DSystem", m_play_modeIn);
  
  // Set OutPort buffer
  addOutPort("ImgOut_2dcdp1", m_img_2dcdp1Out);
  addOutPort("ImgOut_2dcdp2", m_img_2dcdp2Out);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_Control3DShapeServPort.registerConsumer("Creation3DObjResultStatus", "Creation3DObjResultStatusServ::Creation3DObjResultStatus", m_Result3DObjStatusCons);
  
  // Set CORBA Service Ports
  addPort(m_Control3DShapeServPort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // </rtc-template>
  
  // add to code
  std::cout << "onInitialize execution" << std::endl;
  nvd3DShapeCtrlOnInitialize();

  return RTC::RTC_OK;
}

/*!
 * Finalize of the 3DShapeControl RTC
 */

RTC::ReturnCode_t rtc_3DShapeControl::onFinalize()
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t rtc_3DShapeControl::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t rtc_3DShapeControl::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * Activate the 3DShapeControl RTC
 */

RTC::ReturnCode_t rtc_3DShapeControl::onActivated(RTC::UniqueId ec_id)
{
  // add to code
  std::cout << "onActivated execution" << std::endl;
  nvd3DShapeCtrlOnActivated();

  return RTC::RTC_OK;
}

/*!
 * Deactivate the 3DShapeControl RTC
 */

RTC::ReturnCode_t rtc_3DShapeControl::onDeactivated(RTC::UniqueId ec_id)
{
  // add to code
  std::cout << "onDeactivated execution" << std::endl;
  nvd3DShapeCtrlOnDeactivated();

  return RTC::RTC_OK;
}

/*!
 * Periodic task of the 3DShapeControl RTC
 */

RTC::ReturnCode_t rtc_3DShapeControl::onExecute(RTC::UniqueId ec_id)
{
  // add to code
  nvdOnExecuteFunc();

  return RTC::RTC_OK;
}

/*!
 * onAborting the 3DShapeControl RTC
 */

RTC::ReturnCode_t rtc_3DShapeControl::onAborting(RTC::UniqueId ec_id)
{
  // add to code
  std::cout << "onAborting execution" << std::endl;

  return RTC::RTC_OK;
}

/*!
 * onError the 2dcdp RTC
 */
/*
RTC::ReturnCode_t rtc_3DShapeControl::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * Reset initialization of the 3DShapeControl RTC
 */

RTC::ReturnCode_t rtc_3DShapeControl::onReset(RTC::UniqueId ec_id)
{
  // add to code
  std::cout << "onReset execution" << std::endl;

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t rtc_3DShapeControl::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t rtc_3DShapeControl::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/***************************************************************************************************
 *  Function        : nst3DShapeCtrlModeChk1_tbl/nst3DShapeCtrlModeChk2_tbl
 *  Description     : Table definition
 *  Argument        : nothing special
 *  Return          : nothing special
 **************************************************************************************************/
struct _3DShapeCtrlTable_Type
{
  uhword_type uhwNextState;
  uhword_type uhwProcessCode;
};

struct _3DShapeCtrlTable_Type nst3DShapeCtrlModeChk1_tbl[][MTX_01_MODE_MAX] = {
	{ { NO_STATE_CHG,	PROC_NO_STATUS_CHG_MODE       },	{ MTX_MODE_CAM,	PROC_CHG_STATUS_CAMERA          },	{ MTX_MODE_CAM,	PROC_CHG_STATUS_CAMERA      },	{ MTX_MODE_CAM,	PROC_CHG_STATUS_CAMERA      } },
	{ { MTX_MODE_CAP,	PROC_CHG_STATUS_CAPTURE       },	{ NO_STATE_CHG,	PROC_NO_STATUS_CHG_MODE         },	{ MTX_MODE_CAP,	PROC_CHG_STATUS_CAPTURE     },	{ MTX_MODE_CAP,	PROC_CHG_STATUS_CAPTURE     } },
	{ { MTX_MODE_CLD,	PROC_CHG_STATUS_CLOUD         },	{ MTX_MODE_CLD,	PROC_CHG_STATUS_CLOUD           },	{ NO_STATE_CHG,	PROC_NO_STATUS_CHG_MODE     },	{ MTX_MODE_CLD,	PROC_CHG_STATUS_CLOUD       } },
	{ { MTX_MODE_STD,	PROC_CHG_STATUS_STORED        },	{ MTX_MODE_STD,	PROC_CHG_STATUS_STORED          },	{ MTX_MODE_STD,	PROC_CHG_STATUS_STORED      },	{ NO_STATE_CHG,	PROC_NO_STATUS_CHG_MODE     } },
	{ { NO_STATE_CHG,	PROC_3D_IMG_CREATE_ENABLE     },	{ NO_STATE_CHG,	PROC_3D_IMG_CREATE_ENABLE       },	{ NO_STATE_CHG,	PROC_3D_IMG_CREATE_ENABLE   },	{ NO_STATE_CHG,	PROC_3D_IMG_CREATE_ENABLE   } },
	{ { NO_STATE_CHG,	PROC_3D_IMG_CREATE_DISABLE    },	{ NO_STATE_CHG,	PROC_3D_IMG_CREATE_DISABLE      },	{ NO_STATE_CHG,	PROC_3D_IMG_CREATE_DISABLE  },	{ NO_STATE_CHG,	PROC_3D_IMG_CREATE_DISABLE  } },
};

struct _3DShapeCtrlTable_Type nst3DShapeCtrlModeChk2_tbl[][MTX_02_MODE_MAX] = {
	{ { NO_STATE_CHG,	PROC_3D_IMAGE_CREATE_CAMERA   },	{ NO_STATE_CHG,	NO_PROC                         } },
	{ { NO_STATE_CHG,	PROC_3D_IMAGE_CREATE_CAPTURE  },	{ NO_STATE_CHG,	PROC_3D_IMAGE_BACKUP_CAPTURE    } },
	{ { NO_STATE_CHG,	PROC_3D_IMAGE_CREATE_CLOUD    },	{ NO_STATE_CHG,	PROC_3D_IMAGE_BACKUP_CLOUD      } },
	{ { NO_STATE_CHG,	NO_PROC                       },	{ NO_STATE_CHG,	PROC_3D_IMAGE_CREATE_RES_JUDGE  } },
	{ { NO_STATE_CHG,	NO_PROC                       },	{ NO_STATE_CHG,	PROC_3D_IMAGE_CREATE_ERR_JUDGE  } },
};

/***************************************************************************************************
 *  Function        : nstCopyIndex_tbl
 *  Description     : Table definition
 *  Argument        : nothing special
 *  Return          : nothing special
 **************************************************************************************************/
struct copyi_st
{
	ubyte_type des;	//destination
	ubyte_type src;	//source
};

struct copyi_st nstCopyIndex_tbl[][IMAGES_LENGTH_2] = {
	{ { IMG_TPOS_C, IMG_POS_C }, { IMG_TPOS_C, IMG_POS_C } },
	{ { IMG_TPOS_L, IMG_POS_L }, { IMG_TPOS_R, IMG_POS_R } },
};

/***************************************************************************************************
 *  Function        : nvd3DShapeCtrlOnInitialize
 *  Description     : This function the initialization process of the 3D Shape control
 *  Argument        : nothing special
 *  Return          : nothing special
 **************************************************************************************************/
void rtc_3DShapeControl::nvd3DShapeCtrlOnInitialize( void )
{
	nvd3DShapeCtrlOnActivated();

}

/***************************************************************************************************
 *  Function        : nvd3DShapeCtrlOnActivated
 *  Description     : This function the initialization process of the 3D Shape control
 *  Argument        : nothing special
 *  Return          : nothing special
 **************************************************************************************************/
void rtc_3DShapeControl::nvd3DShapeCtrlOnActivated( void )
{
	// input mult-cameraimages
	m_img_camera.images.length( IMAGES_LENGTH_3 );
	m_img_capture.images.length( IMAGES_LENGTH_3 );
	m_img_cloud.images.length( IMAGES_LENGTH_3 );

	// output mult-cameraimages
	m_img_2dcdp1.images.length( IMAGES_LENGTH_2 );
	m_img_2dcdp2.images.length( IMAGES_LENGTH_2 );

	// work mult-cameraimages
	m_img_work.images.length( IMAGES_LENGTH_3 );
	m_img_cap_bkup.images.length( IMAGES_LENGTH_3 );
	m_img_cld_bkup.images.length( IMAGES_LENGTH_3 );

	// variables of matrix
	m_uhwInEventMtx01 = EVENT_MTX01_MAX;
	m_uhwInEventMtx02 = EVENT_MTX02_MAX;
	m_uhwMatrixModeMtx01 = MTX_MODE_CAM;
	m_uhwMatrixModeMtx02 = MTX_IMAGE_UNCREATED;

	// variables of work
	m_uw3DObjResult = SHAPE_OF_3D_INIT;
	m_uw3DObjResultNum = SHAPE_OF_3D_INITIAL;
	m_validity.cap_bkup = FALSE;
	m_validity.cld_bkup = FALSE;
	m_uhwEnableFlag = FALSE;

	m_uhw3DObjMakeErrCnt = TIMER_COUNT_INIT;

}

/***************************************************************************************************
 *  Function        : nvd3DShapeCtrlOnDeactivated
 *  Description     : This function the initialization process of the 3D Shape control
 *  Argument        : nothing special
 *  Return          : nothing special
 **************************************************************************************************/
void rtc_3DShapeControl::nvd3DShapeCtrlOnDeactivated( void )
{
	// input mult-cameraimages
	m_img_camera.images.release();
	m_img_capture.images.release();
	m_img_cloud.images.release();

	// output mult-cameraimages
	m_img_2dcdp1.images.release();
	m_img_2dcdp2.images.release();

	// work mult-cameraimages
	m_img_work.images.release();
	m_img_cap_bkup.images.release();
	m_img_cld_bkup.images.release();

	// variables of matrix
	m_uhwInEventMtx01 = EVENT_MTX01_MAX;
	m_uhwInEventMtx02 = EVENT_MTX02_MAX;
	m_uhwMatrixModeMtx01 = MTX_MODE_CAM;
	m_uhwMatrixModeMtx02 = MTX_IMAGE_UNCREATED;

	// variables of work
	m_uw3DObjResult = SHAPE_OF_3D_INIT;
	m_uw3DObjResultNum = SHAPE_OF_3D_INITIAL;
	m_validity.cap_bkup = FALSE;
	m_validity.cld_bkup = FALSE;
	m_uhwEnableFlag = FALSE;

}

/***************************************************************************************************
 *  Function        : nvdOnExecuteFunc
 *  Description     : This function the main process of the 3D Shape control
 *  Argument        : nothing special
 *  Return          : nothing special
 **************************************************************************************************/
void rtc_3DShapeControl::nvdOnExecuteFunc( void )
{
	nvdOnExecuteMtx01Matrix();
	nvdOnExecuteMtx02Matrix();

}

/***************************************************************************************************
 *  Function        : nvdOnExecuteMtx01Matrix
 *  Description     : This function the main processing of the matrix01
 *  Argument        : sysytem mode(camera/capture/cloud/stored data), play mode(play/stop)
 *  Return          : nothing special
 **************************************************************************************************/
void rtc_3DShapeControl::nvdOnExecuteMtx01Matrix( void )
{
	/* Analyze the input event of the matrix01 */
	uhword_type auhwInEvent = uhw3DShapeCtrlInEvCheckofMtx01();

	if( (auhwInEvent < EVENT_MTX01_MAX) && (m_uhwMatrixModeMtx01 < MTX_01_MODE_MAX) )
	{
		m_uhwInEventMtx01 = auhwInEvent;
		nvd3DShapeCtrlExeOfModeMatrix(m_uhwInEventMtx01, m_uhwMatrixModeMtx01, CASE_MTX01);
	}

}

/***************************************************************************************************
 *  Function        : nvdOnExecuteMtx02Matrix
 *  Description     : This function the main processing of the matrix02
 *  Argument        : image data, 3D image creation inf, 3D image creation error inf
 *  Return          : nothing special
 **************************************************************************************************/
void rtc_3DShapeControl::nvdOnExecuteMtx02Matrix( void )
{
	/* Analyze the input event of the matrix02 */
	uhword_type auhwInEvent = uhw3DShapeCtrlInEvCheckofMtx02();

	if( (auhwInEvent < EVENT_MTX02_MAX) && (m_uhwMatrixModeMtx02 < MTX_02_MODE_MAX) )
	{
		m_uhwInEventMtx02 = auhwInEvent;
		nvd3DShapeCtrlExeOfModeMatrix(m_uhwInEventMtx02, m_uhwMatrixModeMtx02, CASE_MTX02);
  }

}

/***************************************************************************************************
 *  Function        : nvd3DShapeCtrlExeOfModeMatrix
 *  Description     : This function matrix execution of matrix01 or matrix02
 *  Argument        : event, matrix mode
 *  Return          : nothing special
 **************************************************************************************************/
void rtc_3DShapeControl::nvd3DShapeCtrlExeOfModeMatrix( uhword_type auhwInEvent, uhword_type auhwMatrixMode, uhword_type auhwKind )
{
	uhword_type auhwNewState;
	uhword_type auhwProcID;

	uhword_type(rtc_3DShapeControl::*uhwProcess_tbl[])(void) = {
		&rtc_3DShapeControl::uhwProc_NoStatusChangeMode           ,	/*  0	PROC_NO_STATUS_CHG_MODE           */
		&rtc_3DShapeControl::uhwProc_StatusChangeCamera           ,	/*  1	PROC_CHG_STATUS_CAMERA            */
		&rtc_3DShapeControl::uhwProc_StatusChangeCapture          ,	/*  2	PROC_CHG_STATUS_CAPTURE           */
		&rtc_3DShapeControl::uhwProc_StatusChangeCloud            ,	/*  3	PROC_CHG_STATUS_CLOUD             */
		&rtc_3DShapeControl::uhwProc_StatusChangeStoredData       ,	/*  4	PROC_CHG_STATUS_STORED            */
		&rtc_3DShapeControl::uhwProc_3D_Image_Create_Enable       ,	/*  5	PROC_3D_IMG_CREATE_ENABLE         */
		&rtc_3DShapeControl::uhwProc_3D_Image_Create_Disable      ,	/*  6	PROC_3D_IMG_CREATE_DISABLE        */
		&rtc_3DShapeControl::uhwProc_3D_Image_Creation_Camera     ,	/*  7	PROC_3D_IMAGE_CREATE_CAMERA       */
		&rtc_3DShapeControl::uhwProc_3D_Image_Creation_Capture    ,	/*  8	PROC_3D_IMAGE_CREATE_CAPTURE      */
		&rtc_3DShapeControl::uhwProc_3D_Image_Creation_Cloud      ,	/*  9	PROC_3D_IMAGE_CREATE_CLOUD        */
		&rtc_3DShapeControl::uhwProc_3D_Image_Create_Res_Judge    ,	/* 10	PROC_3D_IMAGE_CREATE_RES_JUDGE    */
		&rtc_3DShapeControl::uhwProc_3D_Image_Create_Err_Judge    ,	/* 11	PROC_3D_IMAGE_CREATE_ERR_JUDGE    */
		&rtc_3DShapeControl::uhwProc_3D_Image_BackUp_Capture      ,	/* 12	PROC_3D_IMAGE_BACKUP_CAPTURE      */
		&rtc_3DShapeControl::uhwProc_3D_Image_BackUp_Cloud        ,	/* 13	PROC_3D_IMAGE_BACKUP_CLOUD        */
		&rtc_3DShapeControl::uhwProc_NoProc                       ,	/* 14	PROC_CREATE_SIZE_MAX              */
	};

	if( (auhwKind == CASE_MTX01) || (auhwKind == CASE_MTX02) )
	{
		uhword_type *auhwMatrixMode_p;
		if(auhwKind == CASE_MTX01)
		{
			auhwNewState = nst3DShapeCtrlModeChk1_tbl[auhwInEvent][auhwMatrixMode].uhwNextState;
			auhwProcID 	 = nst3DShapeCtrlModeChk1_tbl[auhwInEvent][auhwMatrixMode].uhwProcessCode;
			auhwMatrixMode_p = &m_uhwMatrixModeMtx01;
		}
		else
		{
			auhwNewState = nst3DShapeCtrlModeChk2_tbl[auhwInEvent][auhwMatrixMode].uhwNextState;
			auhwProcID 	 = nst3DShapeCtrlModeChk2_tbl[auhwInEvent][auhwMatrixMode].uhwProcessCode;
			auhwMatrixMode_p = &m_uhwMatrixModeMtx02;
		}

		if(auhwNewState != NO_STATE_CHG)
		{
			*auhwMatrixMode_p = auhwNewState;
		}

		if(auhwProcID != NO_PROC)
		{
			if( (auhwProcID >= PROC_SIZE_MIN) && (auhwProcID < PROC_SIZE_MAX) )
			{
				auhwNewState = (this->*uhwProcess_tbl[auhwProcID])();
				if(auhwNewState != NO_STATE_CHG)
				{
					*auhwMatrixMode_p = auhwNewState;
				}
			}
		}
	}

}

/***************************************************************************************************
 *  Function        : uhw3DShapeCtrlInEvCheckofMtx01
 *  Description     : This function check the event of the matrix01
 *  Argument        : sysytem mode(camera/capture/cloud/stored data), play mode(play/stop)
 *  Return          : event of the matrix01
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhw3DShapeCtrlInEvCheckofMtx01( void )
{
	uhword_type auhwInEvent = NO_EVENT_INIT;
  std::string astr_m_data = "";

	if( m_sys_modeIn.isNew() )
	{
		m_sys_modeIn.read();
		astr_m_data = m_sys_mode.data;
		auhwInEvent = uhwEventNumSettingOfMtx01(CASE_SYS_MODE, astr_m_data);
	}
	else
	{
		if( m_play_modeIn.isNew() )
		{
			m_play_modeIn.read();
			astr_m_data = m_play_mode.data;
			auhwInEvent = uhwEventNumSettingOfMtx01(CASE_PLY_MODE, astr_m_data);
		}
	}

  return auhwInEvent;

}

/***************************************************************************************************
 *  Function        : uhw3DShapeCtrlInEvCheckofMtx02
 *  Description     : This function check the event of the matrix02
 *  Argument        : image data, 3D image creation inf, 3D image creation error inf
 *  Return          : event of the matrix02
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhw3DShapeCtrlInEvCheckofMtx02( void )
{
	uhword_type auhwInEvent = NO_EVENT_INIT;

	if( (m_uhwMatrixModeMtx02 >= MTX_02_MODE_MIN) && (m_uhwMatrixModeMtx02 < MTX_02_MODE_MAX) )
	{
		if(m_uhwMatrixModeMtx02 == MTX_IMAGE_CREATING)
		{
			auhwInEvent = uhwEventNumSettingOfMtx02TimDet();
		}
		else
		{
			auhwInEvent = uhwEventNumSettingOfMtx02ImgDet();
		}
	}

  return auhwInEvent;

}

/***************************************************************************************************
 *  Function        : uhwEventNumSettingOfMtx01
 *  Description     : event numbers setting of matrix01
 *  Argument        : system mode, play mode
 *  Return          : event of matrix01
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwEventNumSettingOfMtx01(uhword_type auhwKind, std::string as_m_data)
{
	uhword_type auhwInEvent = NO_EVENT_INIT;

	//Lowercase conversion of extension.
	std::transform(as_m_data.begin(), as_m_data.end(), as_m_data.begin(), tolower);

	if( (auhwKind == CASE_SYS_MODE) || (auhwKind == CASE_PLY_MODE) )
	{
		if(auhwKind == CASE_SYS_MODE)
		{
			auhwInEvent = uhwEventNumSettingOfSysMode(as_m_data);
		}
		else
		{
			auhwInEvent = uhwEventNumSettingOfPlyMode(as_m_data);
		}
	}

  return auhwInEvent;
}

/***************************************************************************************************
 *  Function        : uhwEventNumSettingOfSysMode
 *  Description     : event numbers setting of matrix01
 *  Argument        : system mode
 *  Return          : event of matrix01
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwEventNumSettingOfSysMode( std::string as_m_data )
{
	uhword_type auhwInEvent = NO_EVENT_INIT;

	if( (as_m_data.compare("camera") == 0) )
	{
		auhwInEvent = EVENT_SYSTEM_MODE_CAMERA;
	}
	else if( (as_m_data.compare("capture") == 0) )
	{
		auhwInEvent = EVENT_SYSTEM_MODE_CAPTURE;
	}
	else if( (as_m_data.compare("cloud") == 0) )
	{
		auhwInEvent = EVENT_SYSTEM_MODE_CLOUD;
	}
	else if( (as_m_data.compare("storeddata") == 0) )
	{
		auhwInEvent = EVENT_SYSTEM_MODE_STORED_DATA;
	}
	else
	{
	}

  return auhwInEvent;

}

/***************************************************************************************************
 *  Function        : uhwEventNumSettingOfPlyMode
 *  Description     : event numbers setting of matrix01
 *  Argument        : play mode
 *  Return          : event of matrix01
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwEventNumSettingOfPlyMode( std::string as_m_data )
{
	uhword_type auhwInEvent = NO_EVENT_INIT;

	if( (as_m_data.compare("play") == 0) )
	{
		auhwInEvent = EVENT_PLAY_MODE_PLAY;
	}
	else if( (as_m_data.compare("stop") == 0) )
	{
		auhwInEvent = EVENT_PLAY_MODE_STOP;
	}
	else
	{
	}

  return auhwInEvent;

}

/***************************************************************************************************
 *  Function        : uhwEventNumSettingOfMtx02ImgDet
 *  Description     : This function checks the input state of the image
 *  Argument        : port of image data
 *  Return          : event of matrix02
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwEventNumSettingOfMtx02ImgDet( void )
{
	uhword_type auhwInEvent    = NO_EVENT_INIT;
	uhword_type auhwMatrixMode = uhw3DShapeCtrlGetModeOfMtx01();

	if( (auhwMatrixMode == MTX_MODE_CAMERA) && (m_img_cameraIn.isNew() == TRUE) )
	{
		auhwInEvent = EVENT_INPUT_IMAGE_CAMERA;
	}
	else if( (auhwMatrixMode == MTX_MODE_CAPTUR) && (m_img_captureIn.isNew() == TRUE) )
	{
		auhwInEvent = EVENT_INPUT_IMAGE_CAPTURE;
	}
	else if( (auhwMatrixMode == MTX_MODE_DCLOUD) && (m_img_cloudIn.isNew() == TRUE) )
	{
		auhwInEvent = EVENT_INPUT_IMAGE_CLOUD;
	}
	else
	{
	}

  return auhwInEvent;
}

/***************************************************************************************************
 *  Function        : uhwEventNumSettingOfMtx02TimDet
 *  Description     : This function check the event of the timer event
 *  Argument        : nothing special
 *  Return          : event of matrix02
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwEventNumSettingOfMtx02TimDet( void )
{
	uhword_type auhwInEvent = uhwCheck3DObjResultStatus();

	if(auhwInEvent == NO_EVENT_INIT)
	{
		auhwInEvent = uhwEventNumSettingOfTimerError();
		if(auhwInEvent == NO_EVENT_INIT)
		{
			auhwInEvent = uhwCheckInputsImageData();
		}
	}

  return auhwInEvent;
}

/***************************************************************************************************
 *  Function        : uhwEventNumSettingOfTimerError
 *  Description     : This function check the error timer
 *  Argument        : nothing special
 *  Return          : event of matrix02
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwEventNumSettingOfTimerError( void )
{
	uhword_type auhwInEvent = NO_EVENT_INIT;

	if(m_uhw3DObjMakeErrCnt > 0)
	{
		m_uhw3DObjMakeErrCnt = m_uhw3DObjMakeErrCnt - 1;
		if(m_uhw3DObjMakeErrCnt == 0)
		{
			auhwInEvent = EVENT_3D_IMAGE_CREATED_ERROR;
			std::cout << "Failed to create the 3D images-object!" << std::endl;
		}
	}

  return auhwInEvent;

}

/***************************************************************************************************
 *  Function        : uhwCheck3DObjResultStatus
 *  Description     : This function checks the status result of the service port
 *  Argument        : service port
 *  Return          : event of matrix02
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwCheck3DObjResultStatus( void )
{
	uhword_type auhwInEvent = NO_EVENT_INIT;
	uword_type  auw3DObjResult    = SHAPE_OF_3D_INITIAL;
	uword_type  auw3DObjResultNum = SHAPE_OF_3D_INITIAL;

	uword_type auw3DObjResultStatus = uwGetCreation3DObjResultStatus();
	if(auw3DObjResultStatus != SHAPE_OF_3D_INVALID)
	{
		auw3DObjResult = (auw3DObjResultStatus & MASK_RESULT);
		auw3DObjResult = (auw3DObjResult >> MAKE_SHIFT);
		auw3DObjResultNum = (auw3DObjResultStatus & MASK_SEQNUM);

		auhwInEvent = uhwCheck3DObjResultEndAndErr(auw3DObjResult, auw3DObjResultNum);

		if(m_uw3DObjResult != auw3DObjResult)
		{
			m_uw3DObjResult = auw3DObjResult;
		}
	}

  return auhwInEvent;

}

/***************************************************************************************************
 *  Function        : uwGetCreation3DObjResultStatus
 *  Description     : This function gets the status result of the service port
 *  Argument        : service port
 *  Return          : data of service port
 **************************************************************************************************/
uword_type rtc_3DShapeControl::uwGetCreation3DObjResultStatus( void )
{
	uword_type auw3DObjResultStatus = SHAPE_OF_3D_INVALID;

	try{
		if (CORBA::is_nil(m_Result3DObjStatusCons._ptr())){
			std::cout << "[nil] object reference is not assigned." << std::endl;
		}
		else{
			if (m_Result3DObjStatusCons->_non_existent()){
				std::cout << "[inactive] provider is inactive." << std::endl;
			}
			else{
				std::cout << "[active] provider is active." << std::endl;
				auw3DObjResultStatus = m_Result3DObjStatusCons->GetCreation3DObjResultStatus();
			}
		}
	}
	catch (...){
		std::cout << "Unknown exception." << std::endl;
	}

  return auw3DObjResultStatus;

}

/***************************************************************************************************
 *  Function        : uhwCheck3DObjResultEndAndErr
 *  Description     : This function judge the completion of the 3D-image creation
 *  Argument        : 3DObjResultStatus
 *  Return          : event of matrix02
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwCheck3DObjResultEndAndErr( uword_type auw3DObjResult, uword_type auw3DObjResultNum )
{
	uhword_type auhwInEvent = NO_EVENT_INIT;

	nvdBackUp3DObjResultNum(auw3DObjResult, auw3DObjResultNum);

	if( (auw3DObjResult == SHAPE_OF_3D_CREATED) && (m_uw3DObjResult != SHAPE_OF_3D_CREATED) )
	{
		if(m_uw3DObjResultNum == auw3DObjResultNum)
		{
			auhwInEvent = EVENT_3D_IMAGE_CREATED_COMPLETE;
			std::cout << "3D images creation completion!!!" << std::endl;
			m_Result3DObjStatusCons->ReqCreation3DObjResultStatus(SHAPE_OF_3D_INITIAL);
		}
	}
	else if( (auw3DObjResult == SHAPE_OF_3D_ERROR) && (m_uw3DObjResult != SHAPE_OF_3D_ERROR) )
	{
		auhwInEvent = EVENT_3D_IMAGE_CREATED_ERROR;
		std::cout << "Failed to create the 3D images-object!!!" << std::endl;
		m_Result3DObjStatusCons->ReqCreation3DObjResultStatus(SHAPE_OF_3D_INITIAL);
	}
	else
	{
	}

  return auhwInEvent;

}

/***************************************************************************************************
 *  Function        : nvdBackUp3DObjResultNum
 *  Description     : This function judge the completion of the 3D-image creation
 *  Argument        : 3DObjResultStatus
 *  Return          : nothing special
 **************************************************************************************************/
void rtc_3DShapeControl::nvdBackUp3DObjResultNum( uword_type auw3DObjResult, uword_type auw3DObjResultNum )
{
	if( (auw3DObjResult == SHAPE_OF_3D_INITIAL) || (auw3DObjResult == SHAPE_OF_3D_CREATING) )
	{
		m_uw3DObjResultNum = (auw3DObjResultNum & MASK_SEQNUM);
	}
}

/***************************************************************************************************
 *  Function        : uhwCheckInputsImageData
 *  Description     : This function checks the input state of the image
 *  Argument        : port of image data
 *  Return          : event of matrix02
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwCheckInputsImageData( void )
{
	uhword_type auhwInEvent    = NO_EVENT_INIT;
	uhword_type auhwMatrixMode = uhw3DShapeCtrlGetModeOfMtx01();

	if( (auhwMatrixMode == MTX_MODE_CAPTUR) && (m_img_captureIn.isNew() == TRUE) )
	{
		auhwInEvent = EVENT_INPUT_IMAGE_CAPTURE;
	}
	else if( (auhwMatrixMode == MTX_MODE_DCLOUD) && (m_img_cloudIn.isNew() == TRUE) )
	{
		auhwInEvent = EVENT_INPUT_IMAGE_CLOUD;
	}
	else
	{
	}

  return auhwInEvent;

}

/***************************************************************************************************
 *  Function        : uhw3DShapeCtrlGetModeOfMtx01
 *  Description     : This function to get the system mode
 *  Argument        : matrix mode of the mode management
 *  Return          : matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhw3DShapeCtrlGetModeOfMtx01( void )
{
	uhword_type auhwMatrixMode = MTX_MODE_DATA_OF_MAX;

	if(m_uhwMatrixModeMtx01 == MTX_MODE_CAM)
	{
		auhwMatrixMode = MTX_MODE_CAMERA;
	}
	else if(m_uhwMatrixModeMtx01 == MTX_MODE_CAP)
	{
		auhwMatrixMode = MTX_MODE_CAPTUR;
	}
	else if(m_uhwMatrixModeMtx01 == MTX_MODE_CLD)
	{
		auhwMatrixMode = MTX_MODE_DCLOUD;
	}
	else if(m_uhwMatrixModeMtx01 == MTX_MODE_STD)
	{
		auhwMatrixMode = MTX_MODE_STORED;
	}
	else
	{
	}

  return auhwMatrixMode;
}

/***************************************************************************************************
 *  Function        : uhwProc_NoStatusChangeMode
 *  Description     : This function will be executed when there is no change in the status
 *  Argument        : nothing special
 *  Return          : next matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwProc_NoStatusChangeMode( void )
{
	uhword_type auhwRetCode = NO_STATE_CHG;

	#ifdef _DEBUG_SW_
  std::cout << "No status change" << std::endl;

  std::cout << "InEvent:" << m_uhwInEventMtx01 << std::endl;
  std::cout << "Mode:" << m_uhwMatrixModeMtx01 << std::endl;
	#endif

  return auhwRetCode;

}
/***************************************************************************************************
 *  Function        : uhwProc_StatusChangeCamera
 *  Description     : This function will be executed when the status has been changed to camera
 *  Argument        : nothing special
 *  Return          : next matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwProc_StatusChangeCamera( void )
{
	uhword_type auhwRetCode = NO_STATE_CHG;

	#ifdef _DEBUG_SW_
  std::cout << "Status change to camera" << std::endl;

  std::cout << "InEvent:" << m_uhwInEventMtx01 << std::endl;
  std::cout << "Mode:" << m_uhwMatrixModeMtx01 << std::endl;
	#endif

  return auhwRetCode;

}

/***************************************************************************************************
 *  Function        : uhwProc_StatusChangeCapture
 *  Description     : This function will be executed when the status has been changed to capture
 *  Argument        : nothing special
 *  Return          : next matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwProc_StatusChangeCapture( void )
{
	uhword_type auhwRetCode = NO_STATE_CHG;

	#ifdef _DEBUG_SW_
  std::cout << "Status change to capture" << std::endl;

  std::cout << "InEvent:" << m_uhwInEventMtx01 << std::endl;
  std::cout << "Mode:" << m_uhwMatrixModeMtx01 << std::endl;
	#endif

  return auhwRetCode;

}
/***************************************************************************************************
 *  Function        : uhwProc_StatusChangeCloud
 *  Description     : This function will be executed when the status has been changed to cloud
 *  Argument        : nothing special
 *  Return          : next matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwProc_StatusChangeCloud( void )
{
	uhword_type auhwRetCode = NO_STATE_CHG;

	#ifdef _DEBUG_SW_
  std::cout << "Status change to cloud" << std::endl;

  std::cout << "InEvent:" << m_uhwInEventMtx01 << std::endl;
  std::cout << "Mode:" << m_uhwMatrixModeMtx01 << std::endl;
	#endif

  return auhwRetCode;

}
/***************************************************************************************************
 *  Function        : uhwProc_StatusChangeStoredData
 *  Description     : This function will be executed when the status has been changed to stored data
 *  Argument        : nothing special
 *  Return          : next matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwProc_StatusChangeStoredData( void )
{
	uhword_type auhwRetCode = NO_STATE_CHG;

	#ifdef _DEBUG_SW_
  std::cout << "Status change to stored data" << std::endl;

  std::cout << "InEvent:" << m_uhwInEventMtx01 << std::endl;
  std::cout << "Mode:" << m_uhwMatrixModeMtx01 << std::endl;
	#endif

  return auhwRetCode;

}
/***************************************************************************************************
 *  Function        : uhwProc_3D_Image_Create_Enable
 *  Description     : This function will be executed when the status has been changed to play-mode
 *  Argument        : nothing special
 *  Return          : next matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwProc_3D_Image_Create_Enable( void )
{
	uhword_type auhwRetCode = NO_STATE_CHG;

	m_uhwEnableFlag = TRUE;

	#ifdef _DEBUG_SW_
  std::cout << "3D images creation enable" << std::endl;

  std::cout << "InEvent:" << m_uhwInEventMtx01 << std::endl;
  std::cout << "Mode:" << m_uhwMatrixModeMtx01 << std::endl;
	#endif

  return auhwRetCode;

}
/***************************************************************************************************
 *  Function        : uhwProc_3D_Image_Create_Disable
 *  Description     : This function will be executed when the status has been changed to stop-mode
 *  Argument        : nothing special
 *  Return          : next matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwProc_3D_Image_Create_Disable( void )
{
	uhword_type auhwRetCode = NO_STATE_CHG;

	m_uhwEnableFlag = FALSE;

	#ifdef _DEBUG_SW_
  std::cout << "3D images creation disable" << std::endl;

  std::cout << "InEvent:" << m_uhwInEventMtx01 << std::endl;
  std::cout << "Mode:" << m_uhwMatrixModeMtx01 << std::endl;
	#endif

  return auhwRetCode;

}

/***************************************************************************************************
 *  Function        : uhwProc_3D_Image_Creation_Camera
 *  Description     : This function outputs data to 2dcdp
 *  Argument        : multi-cameraimages
 *  Return          : next matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwProc_3D_Image_Creation_Camera( void )
{
	#ifdef _DEBUG_SW_
  std::cout << "3D image creation of camera image" << std::endl;

  std::cout << "InEvent:" << m_uhwInEventMtx02 << std::endl;
  std::cout << "Mode:" << m_uhwMatrixModeMtx02 << std::endl;
	#endif

	m_img_cameraIn.read();
	uhword_type auhwRetCode = uhwJudge3D_Image_Creation( m_img_camera );

	#ifdef _DEBUG_SW_
  std::cout << "auhwRetCode:" << auhwRetCode << std::endl;
	#endif

  return auhwRetCode;

}

/***************************************************************************************************
 *  Function        : uhwProc_3D_Image_Creation_Capture
 *  Description     : This function outputs data to 2dcdp
 *  Argument        : multi-cameraimages
 *  Return          : next matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwProc_3D_Image_Creation_Capture( void )
{
	#ifdef _DEBUG_SW_
  std::cout << "3D image creation of capture image" << std::endl;

  std::cout << "InEvent:" << m_uhwInEventMtx02 << std::endl;
  std::cout << "Mode:" << m_uhwMatrixModeMtx02 << std::endl;
	#endif

	m_img_captureIn.read();
	uhword_type auhwRetCode = uhwJudge3D_Image_Creation( m_img_capture );

	#ifdef _DEBUG_SW_
  std::cout << "auhwRetCode:" << auhwRetCode << std::endl;
	#endif

  return auhwRetCode;

}

/***************************************************************************************************
 *  Function        : uhwProc_3D_Image_Creation_Cloud
 *  Description     : This function outputs data to 2dcdp
 *  Argument        : multi-cameraimages
 *  Return          : next matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwProc_3D_Image_Creation_Cloud( void )
{
	#ifdef _DEBUG_SW_
  std::cout << "3D image creation of cloud image" << std::endl;

  std::cout << "InEvent:" << m_uhwInEventMtx02 << std::endl;
  std::cout << "Mode:" << m_uhwMatrixModeMtx02 << std::endl;
	#endif

	m_img_cloudIn.read();
	uhword_type auhwRetCode = uhwJudge3D_Image_Creation( m_img_cloud );

	#ifdef _DEBUG_SW_
  std::cout << "auhwRetCode:" << auhwRetCode << std::endl;
	#endif

  return auhwRetCode;

}

/***************************************************************************************************
 *  Function        : uhwProc_3D_Image_Create_Res_Judge
 *  Description     : This function judge the completion of the 3D-image creation
 *  Argument        : Backup information of capture image and cloud image
 *  Return          : next matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwProc_3D_Image_Create_Res_Judge( void )
{
	uhword_type auhwRetCode = NO_STATE_CHG;
	uhword_type auhwMatrixMode = uhw3DShapeCtrlGetModeOfMtx01();

	#ifdef _DEBUG_SW_
  std::cout << "Judge the result of image creation" << std::endl;

  std::cout << "InEvent:" << m_uhwInEventMtx02 << std::endl;
  std::cout << "Mode:" << m_uhwMatrixModeMtx02 << std::endl;
	#endif

	if( bJudge3DImagesCreation() )
	{
		if( (auhwMatrixMode == MTX_MODE_CAPTUR) && (m_validity.cap_bkup == TRUE) )
		{
			auhwRetCode = uhw3D_Image_Create_Retry_Judge(&m_img_cap_bkup, BACKUP_CAP_IMG);
		}
		else if( (auhwMatrixMode == MTX_MODE_DCLOUD) && (m_validity.cld_bkup == TRUE) )
		{
			auhwRetCode = uhw3D_Image_Create_Retry_Judge(&m_img_cld_bkup, BACKUP_CLD_IMG);
		}
		else
		{
			auhwRetCode = MTX_IMAGE_UNCREATED;
		}
	}
	else
	{
		auhwRetCode = MTX_IMAGE_UNCREATED;
	}

	#ifdef _DEBUG_SW_
  std::cout << "auhwRetCode:" << auhwRetCode << std::endl;
	#endif

  return auhwRetCode;

}

/***************************************************************************************************
 *  Function        : uhwProc_3D_Image_Create_Err_Judge
 *  Description     : This function judge the completion of the 3D-image creation
 *  Argument        : Backup information of capture image and cloud image
 *  Return          : next matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwProc_3D_Image_Create_Err_Judge( void )
{
	uhword_type auhwRetCode = uhwProc_3D_Image_Create_Res_Judge();

	#ifdef _DEBUG_SW_
  std::cout << "Judge the error of image creation" << std::endl;

  std::cout << "InEvent:" << m_uhwInEventMtx02 << std::endl;
  std::cout << "Mode:" << m_uhwMatrixModeMtx02 << std::endl;
	#endif

	#ifdef _DEBUG_SW_
  std::cout << "auhwRetCode:" << auhwRetCode << std::endl;
	#endif

  return auhwRetCode;

}

/***************************************************************************************************
 *  Function        : uhwProc_3D_Image_BackUp_Capture
 *  Description     : This function backs up the data of the capture-image
 *  Argument        : capture multi-cameraimages
 *  Return          : next matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwProc_3D_Image_BackUp_Capture( void )
{
	uhword_type auhwRetCode = NO_STATE_CHG;

	#ifdef _DEBUG_SW_
  std::cout << "Back up the image of capture" << std::endl;

  std::cout << "InEvent:" << m_uhwInEventMtx02 << std::endl;
  std::cout << "Mode:" << m_uhwMatrixModeMtx02 << std::endl;
	#endif

	m_img_captureIn.read();
	nvd3D_Image_BackUp(&m_img_cap_bkup, m_img_capture, BACKUP_CAP_IMG);

	#ifdef _DEBUG_SW_
  std::cout << "auhwRetCode:" << auhwRetCode << std::endl;
	#endif

  return auhwRetCode;

}

/***************************************************************************************************
 *  Function        : uhwProc_3D_Image_BackUp_Cloud
 *  Description     : This function backs up the data of the cloud-image
 *  Argument        : cloud multi-cameraimages
 *  Return          : next matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwProc_3D_Image_BackUp_Cloud( void )
{
	uhword_type auhwRetCode = NO_STATE_CHG;

	#ifdef _DEBUG_SW_
  std::cout << "Back up the image of cloud" << std::endl;

  std::cout << "InEvent:" << m_uhwInEventMtx02 << std::endl;
  std::cout << "Mode:" << m_uhwMatrixModeMtx02 << std::endl;
	#endif

	m_img_cloudIn.read();
	nvd3D_Image_BackUp(&m_img_cld_bkup, m_img_cloud, BACKUP_CLD_IMG);

	#ifdef _DEBUG_SW_
  std::cout << "auhwRetCode:" << auhwRetCode << std::endl;
	#endif

  return auhwRetCode;

}

/***************************************************************************************************
 *  Function        : uhwProc_NoProc
 *  Description     : Dummy function
 *  Argument        : nothing special
 *  Return          : next matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwProc_NoProc( void )
{
	uhword_type auhwRetCode = NO_STATE_CHG;

  return auhwRetCode;

}

/***************************************************************************************************
 *  Function        : bJudge3DImagesCreation
 *  Description     : This function check the create of 3D image
 *  Argument        : mode of matrix01
 *  Return          : information of normal end or abnormal end
 **************************************************************************************************/
bool_type rtc_3DShapeControl::bJudge3DImagesCreation( void )
{
	bool_type abReturn = FALSE;
	uhword_type auhwMatrixMode = uhw3DShapeCtrlGetModeOfMtx01();

	if( (auhwMatrixMode == MTX_MODE_CAMERA) || (auhwMatrixMode == MTX_MODE_CAPTUR) || (auhwMatrixMode == MTX_MODE_DCLOUD) )
	{
		if( m_uhwEnableFlag == TRUE )
		{
			abReturn = TRUE;
		}
	}

  return abReturn;
}

/***************************************************************************************************
 *  Function        : bCopyMultiCameraImages
 *  Description     : This function validates and copies the Mult-CameraImages data
 *  Argument        : multi-cameraimages
 *  Return          : information of normal end or abnormal end
 **************************************************************************************************/
bool_type rtc_3DShapeControl::bCopyMultiCameraImages(RTC::MultiCameraImages *MultiImage_p, RTC::MultiCameraImages MultiImage)
{
	bool_type abReturn = FALSE;

	if( bCheckMultiCameraImages(MultiImage) )
	{
		abReturn = TRUE;
		nvdCopyMultiCameraImages(MultiImage_p, MultiImage);
	}

  return abReturn;

}

/***************************************************************************************************
 *  Function        : bCheckMultiCameraImages
 *  Description     : This function will check whether the mult-cameraimages data is valid
 *  Argument        : multi-cameraimages
 *  Return          : information of normal end or abnormal end
 **************************************************************************************************/
bool_type rtc_3DShapeControl::bCheckMultiCameraImages(RTC::MultiCameraImages MultiImage)
{
	bool_type abReturn = FALSE;

	uword_type imgwidth;
	uword_type imgheight;
	ubyte_type result[3];

	ubyte_type i = 0;
	for (i = 0; i < MULT_CAMERA_IMG_MAX; i++)
	{
		imgwidth  = MultiImage.images[i].width;
		imgheight = MultiImage.images[i].height;

		result[i] = FALSE;
		if ( bCheckMultiCameraImagesOfSize(imgwidth, imgheight) )
		{
			if(MultiImage.images[i].bpp == FORMAT_COLOR)
			{
				result[i] = TRUE;
			}
		}
	}

	if( (result[0] == TRUE) && (result[1] == TRUE) && (result[2] == TRUE) )
	{
		abReturn = TRUE;
	}

  return abReturn;

}

/***************************************************************************************************
 *  Function        : bCheckMultiCameraImagesOfSize
 *  Description     : This function will check the range of the cameraimage data
 *  Argument        : cameraimage
 *  Return          : information of normal end or abnormal end
 **************************************************************************************************/
bool_type rtc_3DShapeControl::bCheckMultiCameraImagesOfSize(uword_type imgwidth, uword_type imgheight)
{
	bool_type abReturn = FALSE;

	if ((imgwidth == 0) || (imgheight == 0))
	{
		std::cerr << "input camera-image data is empty!" << std::endl;
	}
	else if ((imgwidth <= IMAGE_SIZE_W_MAX) && (imgheight <= IMAGE_SIZE_H_MAX))
	{
		abReturn = TRUE;
	}
	else
	{
		std::cerr << "input camera-image data is oversized!" << std::endl;
	}

  return abReturn;

}

/***************************************************************************************************
 *  Function        : nvdCopyMultiCameraImages
 *  Description     : This function copies the data of Multi-CameraImages
 *  Argument        : multi-cameraimages
 *  Return          : nothing special
 **************************************************************************************************/
void rtc_3DShapeControl::nvdCopyMultiCameraImages(RTC::MultiCameraImages *MultiImage_p, RTC::MultiCameraImages MultiImage)
{
	uword_type imglen;

	ubyte_type i = 0;
	for (i = 0; i < MULT_CAMERA_IMG_MAX; i++)
	{
		imglen = ( ( MultiImage.images[i].width * 3) * MultiImage.images[i].height );
		MultiImage_p->images[i].width  = MultiImage.images[i].width;
		MultiImage_p->images[i].height = MultiImage.images[i].height;
		MultiImage_p->images[i].bpp    = FORMAT_COLOR;
		MultiImage_p->images[i].format = MultiImage.images[i].format;
		MultiImage_p->images[i].fDiv   = 0;
		MultiImage_p->images[i].pixels.length( imglen );
		::memcpy( (void *)&(MultiImage_p->images[i].pixels[0]), (void *)&(MultiImage.images[i].pixels[0]), imglen );
	}

}

/***************************************************************************************************
 *  Function        : bCopyTo2dcdpMultiCameraImages
 *  Description     : This function sets data to be output to 2dcdp1.
 *  Argument        : multi-cameraimages
 *  Return          : nothing special
 **************************************************************************************************/
bool_type rtc_3DShapeControl::bCopyTo2dcdpMultiCameraImages(RTC::MultiCameraImages *MultiImage1_p, RTC::MultiCameraImages *MultiImage2_p, ubyte_type aubKind)
{
	uword_type imglen;
	ubyte_type aubdes = 0;
	ubyte_type aubsrc = 0;
	ubyte_type i      = 0;
	bool_type  abWrtResult = FALSE;

	if( (aubKind == TYPE_2DCDP1) || (aubKind == TYPE_2DCDP2) )
	{
		for (i = 0; i < IMAGES_LENGTH_2; i++)
		{
			aubdes = nstCopyIndex_tbl[i][aubKind].des;
			aubsrc = nstCopyIndex_tbl[i][aubKind].src;

			imglen = ( ( MultiImage2_p->images[aubsrc].width * 3) * MultiImage2_p->images[aubsrc].height );
			MultiImage1_p->images[aubdes].width  = MultiImage2_p->images[aubsrc].width;
			MultiImage1_p->images[aubdes].height = MultiImage2_p->images[aubsrc].height;
			MultiImage1_p->images[aubdes].bpp    = FORMAT_COLOR;
			MultiImage1_p->images[aubdes].format = MultiImage2_p->images[aubsrc].format;
			MultiImage1_p->images[aubdes].fDiv   = 0;
			MultiImage1_p->images[aubdes].pixels.length( imglen );
			::memcpy( (void *)&(MultiImage1_p->images[aubdes].pixels[0]), (void *)&(MultiImage2_p->images[aubsrc].pixels[0]), imglen );
		}

		if( aubKind == TYPE_2DCDP1 )
		{
  		abWrtResult = m_img_2dcdp1Out.write();
		}
		else
		{
  		abWrtResult = m_img_2dcdp2Out.write();
		}

		if(abWrtResult == FALSE)
		{
			std::cerr << "data port is not connected. please check the connect of data port!" << std::endl;
		}
		else
		{
			abWrtResult = TRUE;
		}
	}
	else
	{
		std::cerr << "input parameter is wrong. please check inputs parameter!" << std::endl;
	}

  return abWrtResult;

}

/***************************************************************************************************
 *  Function        : uhwJudge3D_Image_Creation
 *  Description     : This function outputs data to 2dcdp
 *  Argument        : multi-cameraimages
 *  Return          : next matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhwJudge3D_Image_Creation( RTC::MultiCameraImages aimg )
{
	uhword_type auhwRetCode = NO_STATE_CHG;
	bool_type   abWrtRet1   = FALSE;
	bool_type   abWrtRet2   = FALSE;

	if( bJudge3DImagesCreation() )
	{
		if( bCopyMultiCameraImages(&m_img_work, aimg) )
		{
			abWrtRet1 = bCopyTo2dcdpMultiCameraImages(&m_img_2dcdp1, &m_img_work, TYPE_2DCDP1);
			abWrtRet2 = bCopyTo2dcdpMultiCameraImages(&m_img_2dcdp2, &m_img_work, TYPE_2DCDP2);
			if( (abWrtRet1 == TRUE) && (abWrtRet2 == TRUE) )
			{
				auhwRetCode = MTX_IMAGE_CREATING;
				m_uhw3DObjMakeErrCnt = TIMER_COUNT_START;
			}
		}
	}

  return auhwRetCode;

}

/***************************************************************************************************
 *  Function        : nvd3D_Image_BackUp
 *  Description     : This function backs up the data of the capture-image or cloud-image
 *  Argument        : multi-cameraimages
 *  Return          : nothing special
 **************************************************************************************************/
void rtc_3DShapeControl::nvd3D_Image_BackUp( RTC::MultiCameraImages *aimg_p, RTC::MultiCameraImages aimg, uhword_type auhwKind )
{
	uhword_type ahwResult = FALSE;

	if( bJudge3DImagesCreation() )
	{
		if( bCopyMultiCameraImages(aimg_p, aimg) )
		{
			ahwResult = TRUE;
		}

		if( auhwKind == BACKUP_CAP_IMG )
		{
			m_validity.cap_bkup = ahwResult;
		}
		else
		{
			m_validity.cld_bkup = ahwResult;
		}
	}
}

/***************************************************************************************************
 *  Function        : uhw3D_Image_Create_Retry_Judge
 *  Description     : This function judge whether to recreate 3D image
 *  Argument        : multi-cameraimages
 *  Return          : next matrix mode
 **************************************************************************************************/
uhword_type rtc_3DShapeControl::uhw3D_Image_Create_Retry_Judge( RTC::MultiCameraImages *aimg_p, uhword_type auhwKind )
{
	uhword_type auhwRetCode = NO_STATE_CHG;
	uhword_type ahwResult   = FALSE;

	bool_type abWrtRet1 = bCopyTo2dcdpMultiCameraImages(&m_img_2dcdp1, aimg_p, TYPE_2DCDP1);
	bool_type abWrtRet2 = bCopyTo2dcdpMultiCameraImages(&m_img_2dcdp2, aimg_p, TYPE_2DCDP2);

	if( (abWrtRet1 == TRUE) && (abWrtRet2 == TRUE) )
	{
		ahwResult = TRUE;
		m_uhw3DObjMakeErrCnt = TIMER_COUNT_START;
		std::cout << "Recreate the 3D image" << std::endl;
	}
	else
	{
		auhwRetCode = MTX_IMAGE_UNCREATED;
	}

	if( ahwResult == TRUE )
	{
		if( auhwKind == BACKUP_CAP_IMG )
		{
			m_validity.cap_bkup = FALSE;
		}
		else
		{
			m_validity.cld_bkup = FALSE;
		}
	}

  return auhwRetCode;

}

/***************************************************************************************************
 **************************************************************************************************/

extern "C"
{
 
  void rtc_3DShapeControlInit(RTC::Manager* manager)
  {
    coil::Properties profile(rtc_3dshapecontrol_spec);
    manager->registerFactory(profile,
                             RTC::Create<rtc_3DShapeControl>,
                             RTC::Delete<rtc_3DShapeControl>);
  }
  
};


